@@ -92,4 +92,9 @@ unsigned char gt_convert_category(const char *name, int *category);
 
 unsigned char gt_switch_display_mode_value(const unsigned char display_mode, const int protoid, const char device);
 
+grid_type gt_lookup_grid_type(const char *grid_name, const char *module);
+char *gt_get_mps_grid_longname(const grid_type grid, const char *module);
+int gt_lookup_datum_index(const char *datum_str, const char *module);
+char *gt_get_mps_datum_name(const int datum_index);
+
 #endif
