@@ -22,7 +22,7 @@
 #include "defs.h"
 #include "xmlgeneric.h"
 
-static FILE *ofd;
+static gbfile *ofd;
 static waypoint *wpt_tmp;
 static route_head *trk_head;
 
@@ -94,26 +94,26 @@ hiketech_rd_deinit(void)
 static void
 hiketech_wr_init(const char *fname)
 {
-        ofd = xfopen(fname, "w", MYNAME);
+        ofd = gbfopen(fname, "w", MYNAME);
 }
 
 static void
 hiketech_wr_deinit(void)
 {
-        fclose(ofd);
+        gbfclose(ofd);
 }
 
 static void
 hiketech_trk_hdr(const route_head *rte)
 {
-	fprintf(ofd, "<trk>\n");
+	gbfprintf(ofd, "<trk>\n");
 	write_optional_xml_entity(ofd, " ", "ident", rte->rte_name);
 }
 
 static void
 hiketech_trk_tlr(const route_head *rte)
 {
-	fprintf(ofd, "</trk>\n");
+	gbfprintf(ofd, "</trk>\n");
 }
 
 static void
@@ -121,52 +121,52 @@ hiketech_print_utc(time_t tm, const char *indent, const char *tag)
 {
 	char tbuf[80];
         strftime(tbuf, sizeof(tbuf), "%Y-%m-%d %I:%M:%S", gmtime(&tm));
-	fprintf(ofd, "%s<%s>%s</%s>\n",indent,tag,tbuf,tag);
+	gbfprintf(ofd, "%s<%s>%s</%s>\n",indent,tag,tbuf,tag);
 }
 
 static void
 hiketech_trkpt_pr(const waypoint *waypointp)
 {
-	fprintf(ofd, " <pnt>\n");
+	gbfprintf(ofd, " <pnt>\n");
 	if (waypointp->creation_time) {
 		hiketech_print_utc(waypointp->creation_time, "  ", "utc");
 	}
-	fprintf(ofd, "  <lat>%f</lat>\n", waypointp->latitude);
-	fprintf(ofd, "  <long>%f</long>\n", waypointp->longitude);
+	gbfprintf(ofd, "  <lat>%f</lat>\n", waypointp->latitude);
+	gbfprintf(ofd, "  <long>%f</long>\n", waypointp->longitude);
 	if (waypointp->altitude != unknown_alt) {
-		fprintf(ofd, "  <alt>%f</alt>\n",
+		gbfprintf(ofd, "  <alt>%f</alt>\n",
 			 waypointp->altitude);
 	}
-	fprintf(ofd, " </pnt>\n");
+	gbfprintf(ofd, " </pnt>\n");
 }
 
 static void
 hiketech_waypt_pr(const waypoint *wpt)
 {	
-	fprintf(ofd, "<wpt>\n");
+	gbfprintf(ofd, "<wpt>\n");
 	write_xml_entity(ofd, "\t", "ident", wpt->shortname);
 	write_optional_xml_entity(ofd, "\t", "sym", wpt->icon_descr);
-	fprintf(ofd, "\t<lat>%f</lat>\n", wpt->latitude);
-	fprintf(ofd, "\t<long>%f</long>\n", wpt->longitude);
+	gbfprintf(ofd, "\t<lat>%f</lat>\n", wpt->latitude);
+	gbfprintf(ofd, "\t<long>%f</long>\n", wpt->longitude);
 
 	/* 
 	 * These probably aren't technicallyconstants, but it's all 
 	 * we can do for now. 
 	 */
-	fprintf(ofd, "\t<color>\n\t\t<lbl>FAFFB4</lbl>\n\t\t<obj>FF8000</obj>\n\t</color>\n");
-	fprintf(ofd, "</wpt>\n");
+	gbfprintf(ofd, "\t<color>\n\t\t<lbl>FAFFB4</lbl>\n\t\t<obj>FF8000</obj>\n\t</color>\n");
+	gbfprintf(ofd, "</wpt>\n");
 }
 
 static void
 hiketech_write(void)
 {
-	fprintf(ofd, "<hiketech version=\"1.2\" url=\"http://www.hiketech.com\">\n");
-	fprintf(ofd, "<gpsdata>\n");
+	gbfprintf(ofd, "<hiketech version=\"1.2\" url=\"http://www.hiketech.com\">\n");
+	gbfprintf(ofd, "<gpsdata>\n");
 	track_disp_all(hiketech_trk_hdr, hiketech_trk_tlr, hiketech_trkpt_pr);
 	track_disp_all(NULL, NULL, hiketech_trkpt_pr);
 	waypt_disp_all(hiketech_waypt_pr);
-	fprintf(ofd, "</gpsdata>\n");
-	fprintf(ofd, "</hiketech>\n");
+	gbfprintf(ofd, "</gpsdata>\n");
+	gbfprintf(ofd, "</hiketech>\n");
 }
 
 static
