@@ -21,8 +21,7 @@
 
 #include "defs.h"
 #if PDBFMTS_ENABLED
-#include "coldsync/palm.h"
-#include "coldsync/pdb.h"
+#include "pdbfile.h"
 
 #define MYNAME "Companion Waypoints"
 #define MYTYPE  0x54777074  	/* Twpt */
@@ -50,39 +49,36 @@ struct record {
 	char unknown3; /* always 'a' */
 };
 
-static FILE *file_in;
-static FILE *file_out;
-static const char *out_fname;
+static pdbfile *file_in;
+static pdbfile *file_out;
 static short_handle mkshort_handle;
-
-static struct pdb *opdb;
-static struct pdb_record *opdb_rec;
+static int ct;
 
 static void
 rd_init(const char *fname)
 {
-	file_in = xfopen(fname, "rb", MYNAME);
+	file_in = pdb_open(fname, MYNAME);
 }
 
 static void
 rd_deinit(void)
 {
-	fclose(file_in);
+	pdb_close(file_in);
 }
 
 static void
 wr_init(const char *fname)
 {
-	file_out = xfopen(fname, "wb", MYNAME);
-	out_fname = fname;
+	file_out = pdb_create(fname, MYNAME);
 	mkshort_handle = mkshort_new_handle();
 	setshort_length(mkshort_handle, 20);
+	ct = 0;
 }
 
 static void
 wr_deinit(void)
 {
-	fclose(file_out);
+	pdb_close(file_out);
 	mkshort_del_handle(&mkshort_handle);
 }
 
@@ -90,18 +86,13 @@ static void
 data_read(void)
 {
 	struct record *rec;
-	struct pdb *pdb;
-	struct pdb_record *pdb_rec;
+	pdbrec_t *pdb_rec;
 
-	if (NULL == (pdb = pdb_Read(fileno(file_in)))) {
-		fatal(MYNAME ": pdb_Read failed\n");
-	}
-
-	if ((pdb->creator != MYCREATOR) || (pdb->type != MYTYPE)) {
+	if ((file_in->creator != MYCREATOR) || (file_in->type != MYTYPE)) {
 		fatal(MYNAME ": Not a Magellan Navigator file.\n");
 	}
 
-	for(pdb_rec = pdb->rec_index.rec; pdb_rec; pdb_rec=pdb_rec->next) {
+	for(pdb_rec = file_in->rec_list; pdb_rec; pdb_rec = pdb_rec->next) {
 		waypoint *wpt_tmp;
 		char *vdata;
 		struct tm tm;
@@ -133,7 +124,6 @@ data_read(void)
 		waypt_add(wpt_tmp);
 
 	} 
-	free_pdb(pdb);
 }
 
 
@@ -141,7 +131,6 @@ static void
 my_writewpt(const waypoint *wpt)
 {
 	struct record *rec;
-	static int ct;
 	struct tm *tm;
 	char *vdata;
 	time_t tm_t;
@@ -206,15 +195,8 @@ my_writewpt(const waypoint *wpt)
 	vdata[1] = '\0';
 	vdata += 2;
 	
-	opdb_rec = new_Record (0, 0, ct++, (uword) (vdata-(char *)rec), (const ubyte *)rec);
+	pdb_write_rec(file_out, 0, 0, ct++, rec, (char *)vdata - (char *)rec);
 
-	if (opdb_rec == NULL) {
-		fatal(MYNAME ": libpdb couldn't create record\n");
-	}
-
-	if (pdb_AppendRecord(opdb, opdb_rec)) {
-		fatal(MYNAME ": libpdb couldn't append record\n");
-	}
 	xfree(rec);
 }
 
@@ -242,23 +224,16 @@ data_write(void)
 		"\0\x01\x02\x03\x04\x05\x06\x07\x08"
 		"\x09\x0a\x0b\x0c\x0d\x0e\x0f\0\0";
 	
-	if (NULL == (opdb = new_pdb())) { 
-		fatal (MYNAME ": new_pdb failed\n");
-	}
-
-	strncpy(opdb->name, "Companion Waypoints", PDB_DBNAMELEN);
-	opdb->name[PDB_DBNAMELEN-1] = 0;
-	opdb->attributes = PDB_ATTR_BACKUP;
-	opdb->ctime = opdb->mtime = current_time() + 2082844800U;
-	opdb->type = MYTYPE;  /* CWpt */
-	opdb->creator = MYCREATOR; /* cGPS */
-	opdb->version = 1;
-	opdb->appinfo = (void *)appinfo;
-	opdb->appinfo_len = 276;
+	strncpy(file_out->name, "Companion Waypoints", PDB_DBNAMELEN);
+	file_out->attr = PDB_FLAG_BACKUP;
+	file_out->ctime = file_out->mtime = current_time() + 2082844800U;
+	file_out->type = MYTYPE;  /* CWpt */
+	file_out->creator = MYCREATOR; /* cGPS */
+	file_out->version = 1;
+	file_out->appinfo = (void *)appinfo;
+	file_out->appinfo_len = 276;
 
 	waypt_disp_all(my_writewpt);
-	
-	pdb_Write(opdb, fileno(file_out));
 }
 
 
